# Copyright 2020 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class NisporRouteRuleState:
    def __init__(self, info):
        self._rls = [NisporRouteRule(rl_info) for rl_info in info]

    def __iter__(self):
        for rl in self._rls:
            yield rl


class NisporRouteRule:
    def __init__(self, info):
        self._info = info

    @property
    def action(self):
        return self._info["action"]

    @property
    def address_family(self):
        return self._info["address_family"]

    @property
    def flags(self):
        return self._info["flags"]

    @property
    def tos(self):
        return self._info["tos"]

    @property
    def table(self):
        return self._info.get("table")

    @property
    def dst(self):
        return self._info.get("dst")

    @property
    def src(self):
        return self._info.get("src")

    @property
    def iif(self):
        return self._info.get("iif")

    @property
    def oif(self):
        return self._info.get("oif")

    @property
    def goto(self):
        return self._info.get("goto")

    @property
    def priority(self):
        return self._info.get("priority")

    @property
    def fw_mark(self):
        return self._info.get("fw_mark")

    @property
    def fw_mask(self):
        return self._info.get("fw_mask")

    @property
    def mask(self):
        return self._info.get("mask")

    @property
    def flow(self):
        return self._info.get("flow")

    @property
    def tun_id(self):
        return self._info.get("tun_id")

    @property
    def suppress_ifgroup(self):
        return self._info.get("suppress_ifgroup")

    @property
    def suppress_prefix_len(self):
        return self._info.get("suppress_prefix_len")

    @property
    def protocol(self):
        return self._info.get("protocol")

    @property
    def ip_proto(self):
        return self._info.get("ip_proto")

    @property
    def src_port_range(self):
        return self._info.get("src_port_range")

    @property
    def dst_port_range(self):
        return self._info.get("dst_port_range")

    @property
    def l3mdev(self):
        return self._info.get("l3mdev", False)



# Copyright 2020 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class NisporSriov:
    def __init__(self, info):
        self._vfs = [NisporSriovVf(vf_info) for vf_info in info["vfs"]]

    @property
    def vfs(self):
        return self._vfs


class NisporSriovVf:
    def __init__(self, info):
        self._info = info

    @property
    def vf_id(self):
        return self._info["id"]

    @property
    def mac(self):
        return self._info["mac"]

    @property
    def broadcast(self):
        return self._info["broadcast"]

    @property
    def vlan_id(self):
        return self._info["vlan_id"]

    @property
    def qos(self):
        return self._info["qos"]

    @property
    def tx_rate(self):
        return self._info["tx_rate"]

    @property
    def spoof_check(self):
        return self._info["spoof_check"]

    @property
    def link_state(self):
        return self._info["link_state"]

    @property
    def min_tx_rate(self):
        return self._info["min_tx_rate"]

    @property
    def max_tx_rate(self):
        return self._info["max_tx_rate"]

    @property
    def query_rss(self):
        return self._info["query_rss"]

    @property
    def state(self):
        return NisporSriovVfState(self._info["state"])

    @property
    def trust(self):
        return self._info["trust"]

    @property
    def ib_node_guid(self):
        return self._info.get("ib_node_guid")

    @property
    def ib_port_guid(self):
        return self._info.get("ib_port_guid")


class NisporSriovVfState:
    def __init__(self, info):
        self._info = info

    @property
    def rx_packets(self):
        return self._info["rx_packets"]

    @property
    def tx_packets(self):
        return self._info["tx_packets"]

    @property
    def rx_bytes(self):
        return self._info["rx_bytes"]

    @property
    def tx_bytes(self):
        return self._info["tx_bytes"]

    @property
    def broadcast(self):
        return self._info["broadcast"]

    @property
    def multicast(self):
        return self._info["multicast"]

    @property
    def rx_dropped(self):
        return self._info["rx_dropped"]

    @property
    def tx_dropped(self):
        return self._info["tx_dropped"]



# Copyright 2020 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .base_iface import NisporBaseIface
from .bond import NisporBond
from .bond import NisporBondSubordinate
from .bridge import NisporBridge
from .bridge import NisporBridgePort
from .clib_wrapper import NisporError
from .clib_wrapper import retrieve_net_state_json
from .iface import NisporIfaceState
from .route import NisporRoute
from .route import NisporRouteState
from .route_rule import NisporRouteRule
from .route_rule import NisporRouteRuleState
from .state import NisporNetState
from .tun import NisporTun
from .veth import NisporVeth
from .vlan import NisporVlan
from .vxlan import NisporVxlan


__all__ = []



# Copyright 2020 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .base_iface import NisporBaseIface


class NisporVlan(NisporBaseIface):
    def __init__(self, info):
        super().__init__(info)
        self._vlan_info = self._info.get("vlan", {})

    @property
    def vlan_id(self):
        return self._vlan_info.get("vlan_id")

    @property
    def protocol(self):
        return self._vlan_info.get("protocol")

    @property
    def base_iface(self):
        return self._vlan_info.get("base_iface")

    @property
    def is_reorder_hdr(self):
        return self._vlan_info.get("is_reorder_hdr")

    @property
    def is_gvrp(self):
        return self._vlan_info.get("is_gvrp")

    @property
    def is_loose_binding(self):
        return self._vlan_info.get("is_loose_binding")

    @property
    def is_mvrp(self):
        return self._vlan_info.get("is_mvrp")

    @property
    def is_bridge_binding(self):
        return self._vlan_info.get("is_bridge_binding")



# Copyright 2020 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class NisporIPAddr:
    def __init__(self, info):
        self._info = info

    @property
    def address(self):
        return self._info["address"]

    @property
    def prefix_len(self):
        return self._info["prefix_len"]

    @property
    def valid_lft(self):
        return self._info["valid_lft"]

    @property
    def preferred_lft(self):
        return self._info["preferred_lft"]


class NisporIPv4Addr(NisporIPAddr):
    @property
    def peer(self):
        return self._info.get("peer")


class NisporIPv6Addr(NisporIPAddr):
    pass


class NisporIPv4:
    def __init__(self, info):
        self._info = info
        self._address = []
        for addr_info in info.get("addresses", []):
            self._address.append(NisporIPv4Addr(addr_info))

    @property
    def addresses(self):
        return self._address

    def __str__(self):
        return f"{self._info}"


class NisporIPv6:
    def __init__(self, info):
        self._info = info
        self._address = []
        for addr_info in info.get("addresses", []):
            self._address.append(NisporIPv6Addr(addr_info))

    @property
    def addresses(self):
        return self._address

    def __str__(self):
        return f"{self._info}"



# Copyright 2020 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .ip import NisporIPv4
from .ip import NisporIPv6
from .sr_iov import NisporSriov
from .ethtool import NisporEthtool


class NisporBaseIface:
    def __init__(self, info):
        self._info = info
        self._sub_state = None
        self._sr_iov = None
        self._ethtool = None
        if "sriov" in self._info:
            self._sr_iov = NisporSriov(self._info["sriov"])
        if "ethtool" in self._info:
            self._ethtool = NisporEthtool(self._info["ethtool"])

    def __str__(self):
        return f"{self._info}"

    @property
    def type(self):
        return self._info["iface_type"]

    @property
    def name(self):
        return self._info["name"]

    @property
    def state(self):
        return self._info["state"]

    @property
    def mtu(self):
        return self._info["mtu"]

    @property
    def flags(self):
        return self._info["flags"]

    @property
    def ipv4(self):
        if "ipv4" in self._info:
            return NisporIPv4(self._info["ipv4"])
        else:
            return None

    @property
    def ipv6(self):
        if "ipv6" in self._info:
            return NisporIPv6(self._info["ipv6"])
        else:
            return None

    @property
    def mac_address(self):
        return self._info.get("mac_address")

    @property
    def permanent_mac_address(self):
        return self._info.get("permanent_mac_address")

    @property
    def controller(self):
        return self._info.get("controller")

    @property
    def controller_type(self):
        return self._info.get("controller_type")

    @property
    def subordinate_state(self):
        return self._sub_state

    @subordinate_state.setter
    def subordinate_state(self, value):
        self._sub_state = value

    @property
    def sr_iov(self):
        return self._sr_iov

    @property
    def ethtool(self):
        return self._ethtool


class NisporBaseSubordinateIface:
    def __init__(self):
        self._sub_info = None

    def __str__(self):
        return f"{self._sub_info}"



# Copyright 2020 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class NisporRouteState:
    def __init__(self, info):
        self._rts = [NisporRoute(rt_info) for rt_info in info]

    def __iter__(self):
        for rt in self._rts:
            yield rt


class NisporRoute:
    def __init__(self, info):
        self._info = info

    @property
    def address_family(self):
        return self._info["address_family"]

    @property
    def tos(self):
        return self._info["tos"]

    @property
    def table(self):
        return self._info["table"]

    @property
    def protocol(self):
        return self._info["protocol"]

    @property
    def scope(self):
        return self._info["scope"]

    @property
    def route_type(self):
        return self._info["route_type"]

    @property
    def flags(self):
        return self._info["flags"]

    @property
    def dst(self):
        return self._info.get("dst")

    @property
    def oif(self):
        return self._info.get("oif")

    @property
    def iif(self):
        return self._info.get("iif")

    @property
    def prefered_src(self):
        return self._info.get("prefered_src")

    @property
    def src(self):
        return self._info.get("src")

    @property
    def class_id(self):
        return self._info.get("class_id")

    @property
    def gateway(self):
        return self._info.get("gateway")

    @property
    def via(self):
        return self._info.get("via")

    @property
    def mark(self):
        return self._info.get("mark")

    @property
    def uid(self):
        return self._info.get("uid")

    @property
    def lock(self):
        return self._info.get("lock")

    @property
    def mtu(self):
        return self._info.get("mtu")

    @property
    def window(self):
        return self._info.get("window")

    @property
    def rtt(self):
        return self._info.get("rtt")

    @property
    def rttvar(self):
        return self._info.get("rttvar")

    @property
    def ssthresh(self):
        return self._info.get("ssthresh")

    @property
    def cwnd(self):
        return self._info.get("cwnd")

    @property
    def advmss(self):
        return self._info.get("advmss")

    @property
    def reordering(self):
        return self._info.get("reordering")

    @property
    def hoplimit(self):
        return self._info.get("hoplimit")

    @property
    def initcwnd(self):
        return self._info.get("initcwnd")

    @property
    def features(self):
        return self._info.get("features")

    @property
    def rto_min(self):
        return self._info.get("rto_min")

    @property
    def initrwnd(self):
        return self._info.get("initrwnd")

    @property
    def quickack(self):
        return self._info.get("quickack")

    @property
    def cc_algo(self):
        return self._info.get("cc_algo")

    @property
    def fastopen_no_cookie(self):
        return self._info.get("fastopen_no_cookie")

    @property
    def cache_clntref(self):
        return self._info.get("cache_clntref")

    @property
    def cache_last_use(self):
        return self._info.get("cache_last_use")

    @property
    def cache_expires(self):
        return self._info.get("cache_expires")

    @property
    def cache_error(self):
        return self._info.get("cache_error")

    @property
    def cache_used(self):
        return self._info.get("cache_used")

    @property
    def cache_id(self):
        return self._info.get("cache_id")

    @property
    def cache_ts(self):
        return self._info.get("cache_ts")

    @property
    def cache_ts_age(self):
        return self._info.get("cache_ts_age")

    @property
    def metric(self):
        return self._info.get("metric")

    @property
    def perf(self):
        return self._info.get("perf")



# Copyright 2020 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import ctypes
from ctypes import c_int, c_char_p, c_uint32, Structure, POINTER, byref
from ctypes.util import find_library

lib = ctypes.cdll.LoadLibrary(find_library("nispor"))

lib.nispor_net_state_retrieve.restype = c_int
lib.nispor_net_state_retrieve.argtypes = (
    POINTER(c_char_p),
    POINTER(c_char_p),
    POINTER(c_char_p),
)
lib.nispor_net_state_free.restype = None
lib.nispor_net_state_free.argtypes = (c_char_p,)
lib.nispor_err_kind_free.restype = None
lib.nispor_err_kind_free.argtypes = (c_char_p,)
lib.nispor_err_msg_free.restype = None
lib.nispor_err_msg_free.argtypes = (c_char_p,)


class NisporError(Exception):
    def __init__(self, kind, msg):
        self.kind = kind
        self.msg = msg
        super().__init__(f"{kind}: {msg}")


def retrieve_net_state_json():
    c_err_msg = c_char_p()
    c_err_kind = c_char_p()
    c_state = c_char_p()
    rc = lib.nispor_net_state_retrieve(
        byref(c_state), byref(c_err_kind), byref(c_err_msg)
    )
    state = c_state.value
    err_msg = c_err_msg.value
    err_kind = c_err_kind.value
    lib.nispor_net_state_free(c_state)
    lib.nispor_err_kind_free(c_err_kind)
    lib.nispor_err_msg_free(c_err_msg)
    if rc != 0:
        raise NisporError(err_kind, err_msg)
    return state.decode("utf-8")



# Copyright 2020 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json

from .clib_wrapper import retrieve_net_state_json
from .iface import NisporIfaceState
from .route import NisporRouteState
from .route_rule import NisporRouteRuleState


class NisporNetState:
    def __str__(self):
        return f"{self._info}"

    def __init__(self, info):
        self._info = info
        self._ifaces = NisporIfaceState(info.get("ifaces"))
        self._routes = NisporRouteState(info.get("routes"))
        self._route_rules = NisporRouteRuleState(info.get("rules"))

    @property
    def ifaces(self):
        return self._ifaces

    @property
    def routes(self):
        return self._routes

    @property
    def route_rules(self):
        return self._route_rules

    @staticmethod
    def retrieve():
        return NisporNetState(json.loads(retrieve_net_state_json()))



# Copyright 2020 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .base_iface import NisporBaseIface


class NisporTun(NisporBaseIface):
    def __init__(self, info):
        super().__init__(info)
        self._tun_info = self._info.get("tun", {})

    @property
    def mode(self):
        return self._tun_info["mode"]

    @property
    def owner(self):
        return self._tun_info.get("owner")

    @property
    def group(self):
        return self._tun_info.get("group")

    @property
    def pi(self):
        return self._tun_info["pi"]

    @property
    def vnet_hdr(self):
        return self._tun_info["vnet_hdr"]

    @property
    def multi_queue(self):
        return self._tun_info["multi_queue"]

    @property
    def persist(self):
        return self._tun_info["persist"]

    @property
    def num_queues(self):
        return self._tun_info.get("num_queues")

    @property
    def num_disabled_queues(self):
        return self._tun_info.get("num_disabled_queues")



import setuptools

setuptools.setup(
    name="nispor",
    version="1.1.0",
    author="Gris Ge",
    author_email="fge@redhat.com",
    description="Python binding of Nispor",
    long_description="Python binding of Nipor for kernel querying network state",
    url="https://github.com/nispor/nispor/",
    packages=setuptools.find_packages(),
    license="ASL2.0+",
    python_requires='>=3.6',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: POSIX :: Linux",
    ],
)




# Copyright 2020 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .base_iface import NisporBaseIface
from .base_iface import NisporBaseSubordinateIface


class NisporBond(NisporBaseIface):
    def __init__(self, info):
        super().__init__(info)
        self._bond_info = self._info.get("bond", {})

    @property
    def subordinates(self):
        return self._bond_info["subordinates"]

    @property
    def mode(self):
        return self._bond_info["mode"]

    @property
    def miimon(self):
        return self._bond_info.get("miimon")

    @property
    def updelay(self):
        return self._bond_info.get("updelay")

    @property
    def downdelay(self):
        return self._bond_info.get("downdelay")

    @property
    def use_carrier(self):
        return self._bond_info.get("use_carrier")

    @property
    def arp_interval(self):
        return self._bond_info.get("arp_interval")

    @property
    def arp_ip_target(self):
        return self._bond_info.get("arp_ip_target")

    @property
    def arp_all_targets(self):
        return self._bond_info.get("arp_all_targets")

    @property
    def arp_validate(self):
        return self._bond_info.get("arp_validate")

    @property
    def primary(self):
        return self._bond_info.get("primary")

    @property
    def primary_reselect(self):
        return self._bond_info.get("primary_reselect")

    @property
    def fail_over_mac(self):
        return self._bond_info.get("fail_over_mac")

    @property
    def xmit_hash_policy(self):
        return self._bond_info.get("xmit_hash_policy")

    @property
    def resend_igmp(self):
        return self._bond_info.get("resend_igmp")

    @property
    def num_unsol_na(self):
        return self._bond_info.get("num_unsol_na")

    @property
    def num_grat_arp(self):
        return self._bond_info.get("num_grat_arp")

    @property
    def all_subordinates_active(self):
        return self._bond_info.get("all_subordinates_active")

    @property
    def min_links(self):
        return self._bond_info.get("min_links")

    @property
    def lp_interval(self):
        return self._bond_info.get("lp_interval")

    @property
    def packets_per_subordinate(self):
        return self._bond_info.get("packets_per_subordinate")

    @property
    def lacp_rate(self):
        return self._bond_info.get("lacp_rate")

    @property
    def ad_select(self):
        return self._bond_info.get("ad_select")

    @property
    def ad_actor_sys_prio(self):
        return self._bond_info.get("ad_actor_sys_prio")

    @property
    def ad_user_port_key(self):
        return self._bond_info.get("ad_user_port_key")

    @property
    def ad_actor_system(self):
        return self._bond_info.get("ad_actor_system")

    @property
    def tlb_dynamic_lb(self):
        return self._bond_info.get("tlb_dynamic_lb")

    @property
    def peer_notif_delay(self):
        return self._bond_info.get("peer_notif_delay")

    @property
    def ad_info(self):
        return self._bond_info.get("ad_info")


class NisporBondSubordinate(NisporBaseSubordinateIface):
    def __init__(self, info):
        super().__init__()
        self._sub_info = info.get("bond_subordinate", {})

    @property
    def subordinate_state(self):
        return self._sub_info.get("subordinate_state")

    @property
    def mii_status(self):
        return self._sub_info.get("mii_status")

    @property
    def link_failure_count(self):
        return self._sub_info.get("link_failure_count")

    @property
    def perm_hwaddr(self):
        return self._sub_info.get("perm_hwaddr")

    @property
    def queue_id(self):
        return self._sub_info.get("queue_id")

    @property
    def ad_aggregator_id(self):
        return self._sub_info.get("ad_aggregator_id")

    @property
    def ad_actor_oper_port_state(self):
        return self._sub_info.get("ad_actor_oper_port_state")

    @property
    def ad_partner_oper_port_state(self):
        return self._sub_info.get("ad_partner_oper_port_state")



# Copyright 2020 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .base_iface import NisporBaseIface
from .base_iface import NisporBaseSubordinateIface


class NisporBridge(NisporBaseIface):
    def __init__(self, info):
        super().__init__(info)
        self._br_info = self._info.get("bridge")

    @property
    def ports(self):
        return self._br_info.get("ports")

    @property
    def subordinates(self):
        return self.ports

    @property
    def options(self):
        if self._br_info:
            return {
                key: value
                for key, value in self._br_info.items()
                if key != "ports"
            }
        return None


class NisporBridgePortVlan:
    def __init__(
        self, vid=None, vid_range=None, is_pvid=None, is_egress_untagged=None
    ):
        self.vid = vid
        self.vid_range = vid_range
        self.is_pvid = is_pvid
        self.is_egress_untagged = is_egress_untagged

    def __str__(self):
        return f"{self.__dict__}"


class NisporBridgePort(NisporBaseSubordinateIface):
    def __init__(self, info):
        super().__init__()
        self._sub_info = info.get("bridge_port")
        self._vlans = None
        if self._sub_info:
            self._vlans = []
            for vlan in self._sub_info.get("vlans", []):
                self._vlans.append(
                    NisporBridgePortVlan(
                        vlan.get("vid"),
                        vlan.get("vid_range"),
                        vlan.get("is_pvid"),
                        vlan.get("is_egress_untagged"),
                    )
                )

    @property
    def stp_state(self):
        return self._sub_info.get("stp_state")

    @property
    def stp_priority(self):
        return self._sub_info.get("stp_priority")

    @property
    def stp_path_cost(self):
        return self._sub_info.get("stp_path_cost")

    @property
    def hairpin_mode(self):
        return self._sub_info.get("hairpin_mode")

    @property
    def bpdu_guard(self):
        return self._sub_info.get("bpdu_guard")

    @property
    def root_block(self):
        return self._sub_info.get("root_block")

    @property
    def multicast_fast_leave(self):
        return self._sub_info.get("multicast_fast_leave")

    @property
    def learning(self):
        return self._sub_info.get("learning")

    @property
    def unicast_flood(self):
        return self._sub_info.get("unicast_flood")

    @property
    def proxyarp(self):
        return self._sub_info.get("proxyarp")

    @property
    def proxyarp_wifi(self):
        return self._sub_info.get("proxyarp_wifi")

    @property
    def designated_root(self):
        return self._sub_info.get("designated_root")

    @property
    def designated_bridge(self):
        return self._sub_info.get("designated_bridge")

    @property
    def designated_port(self):
        return self._sub_info.get("designated_port")

    @property
    def designated_cost(self):
        return self._sub_info.get("designated_cost")

    @property
    def port_id(self):
        return self._sub_info.get("port_id")

    @property
    def port_no(self):
        return self._sub_info.get("port_no")

    @property
    def change_ack(self):
        return self._sub_info.get("change_ack")

    @property
    def config_pending(self):
        return self._sub_info.get("config_pending")

    @property
    def message_age_timer(self):
        return self._sub_info.get("message_age_timer")

    @property
    def forward_delay_timer(self):
        return self._sub_info.get("forward_delay_timer")

    @property
    def hold_timer(self):
        return self._sub_info.get("hold_timer")

    @property
    def multicast_router(self):
        return self._sub_info.get("multicast_router")

    @property
    def multicast_flood(self):
        return self._sub_info.get("multicast_flood")

    @property
    def multicast_to_unicast(self):
        return self._sub_info.get("multicast_to_unicast")

    @property
    def vlan_tunnel(self):
        return self._sub_info.get("vlan_tunnel")

    @property
    def broadcast_flood(self):
        return self._sub_info.get("broadcast_flood")

    @property
    def group_fwd_mask(self):
        return self._sub_info.get("group_fwd_mask")

    @property
    def neigh_suppress(self):
        return self._sub_info.get("neigh_suppress")

    @property
    def isolated(self):
        return self._sub_info.get("isolated")

    @property
    def backup_port(self):
        return self._sub_info.get("backup_port")

    @property
    def vlans(self):
        return self._vlans



# Copyright 2020 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .base_iface import NisporBaseIface


class NisporVxlan(NisporBaseIface):
    def __init__(self, info):
        super().__init__(info)
        self._vxlan_info = self._info.get("vxlan", {})

    @property
    def vlan_id(self):
        return self._vlan_info.get("vlan_id")

    @property
    def remote(self):
        return self._vxlan_info.get("remote")

    @property
    def vxlan_id(self):
        return self._vxlan_info.get("vxlan_id")

    @property
    def base_iface(self):
        return self._vxlan_info.get("base_iface")

    @property
    def local(self):
        return self._vxlan_info.get("local")

    @property
    def ttl(self):
        return self._vxlan_info.get("ttl")

    @property
    def tos(self):
        return self._vxlan_info.get("tos")

    @property
    def learning(self):
        return self._vxlan_info.get("learning")

    @property
    def ageing(self):
        return self._vxlan_info.get("ageing")

    @property
    def max_address(self):
        return self._vxlan_info.get("max_address")

    @property
    def src_port_min(self):
        return self._vxlan_info.get("src_port_min")

    @property
    def src_port_max(self):
        return self._vxlan_info.get("src_port_max")

    @property
    def proxy(self):
        return self._vxlan_info.get("proxy")

    @property
    def rsc(self):
        return self._vxlan_info.get("rsc")

    @property
    def l2miss(self):
        return self._vxlan_info.get("l2miss")

    @property
    def l3miss(self):
        return self._vxlan_info.get("l3miss")

    @property
    def dst_port(self):
        return self._vxlan_info.get("dst_port")

    @property
    def dst_port_min(self):
        # Deprecated. Will remove in 0.7.0
        return self._vxlan_info.get("dst_port")

    @property
    def dst_port_max(self):
        # Deprecated. Will remove in 0.7.0
        return self._vxlan_info.get("dst_port")

    @property
    def udp_check_sum(self):
        return self._vxlan_info.get("udp_check_sum")

    @property
    def udp6_zero_check_sum_tx(self):
        return self._vxlan_info.get("udp6_zero_check_sum_tx")

    @property
    def udp6_zero_check_sum_rx(self):
        return self._vxlan_info.get("udp6_zero_check_sum_rx")

    @property
    def remote_check_sum_tx(self):
        return self._vxlan_info.get("remote_check_sum_tx")

    @property
    def remote_check_sum_rx(self):
        return self._vxlan_info.get("remote_check_sum_rx")

    @property
    def gbp(self):
        return self._vxlan_info.get("gbp")

    @property
    def remote_check_sum_no_partial(self):
        return self._vxlan_info.get("remote_check_sum_no_partial")

    @property
    def collect_metadata(self):
        return self._vxlan_info.get("collect_metadata")

    @property
    def label(self):
        return self._vxlan_info.get("label")

    @property
    def gpe(self):
        return self._vxlan_info.get("gpe")

    @property
    def ttl_inherit(self):
        return self._vxlan_info.get("ttl_inherit")

    @property
    def df(self):
        return self._vxlan_info.get("df")



# Copyright 2020-2021 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class NisporEthtool:
    def __init__(self, info):
        if "pause" in info:
            self._pause = NisporEthtoolPause(info["pause"])
        else:
            self._pause = None

        if "features" in info:
            self._features = NisporEthtoolFeatures(info["features"])
        else:
            self._features = None

        if "coalesce" in info:
            self._coalesce = NisporEthtoolCoalesce(info["coalesce"])
        else:
            self._coalesce = None

        if "ring" in info:
            self._ring = NisporEthtoolRing(info["ring"])
        else:
            self._ring = None

        if "link_mode" in info:
            self._link_mode = NisporEthtoolLinkMode(info["link_mode"])
        else:
            self._link_mode = None

    @property
    def pause(self):
        return self._pause

    @property
    def features(self):
        return self._features

    @property
    def coalesce(self):
        return self._coalesce

    @property
    def ring(self):
        return self._ring

    @property
    def link_mode(self):
        return self._link_mode

class NisporEthtoolPause:
    def __init__(self, info):
        self._info = info

    @property
    def rx(self):
        return self._info["rx"]

    @property
    def tx(self):
        return self._info["tx"]

    @property
    def auto_negotiate(self):
        return self._info["auto_negotiate"]


class NisporEthtoolFeatures:
    def __init__(self, info):
        self._info = info

    @property
    def fixed(self):
        return self._info["fixed"]

    @property
    def changeable(self):
        return self._info["changeable"]


class NisporEthtoolCoalesce:
    def __init__(self, info):
        self._info = info

    @property
    def pkt_rate_high(self):
        return self._info.get("pkt_rate_high")

    @property
    def pkt_rate_low(self):
        return self._info.get("pkt_rate_low")

    @property
    def rate_sample_interval(self):
        return self._info.get("rate_sample_interval")

    @property
    def rx_max_frames(self):
        return self._info.get("rx_max_frames")

    @property
    def rx_max_frames_high(self):
        return self._info.get("rx_max_frames_high")

    @property
    def rx_max_frames_irq(self):
        return self._info.get("rx_max_frames_irq")

    @property
    def rx_max_frames_low(self):
        return self._info.get("rx_max_frames_low")

    @property
    def rx_usecs(self):
        return self._info.get("rx_usecs")

    @property
    def rx_usecs_high(self):
        return self._info.get("rx_usecs_high")

    @property
    def rx_usecs_irq(self):
        return self._info.get("rx_usecs_irq")

    @property
    def rx_usecs_low(self):
        return self._info.get("rx_usecs_low")

    @property
    def stats_block_usecs(self):
        return self._info.get("stats_block_usecs")

    @property
    def tx_max_frames(self):
        return self._info.get("tx_max_frames")

    @property
    def tx_max_frames_high(self):
        return self._info.get("tx_max_frames_high")

    @property
    def tx_max_frames_irq(self):
        return self._info.get("tx_max_frames_irq")

    @property
    def tx_max_frames_low(self):
        return self._info.get("tx_max_frames_low")

    @property
    def tx_usecs(self):
        return self._info.get("tx_usecs")

    @property
    def tx_usecs_high(self):
        return self._info.get("tx_usecs_high")

    @property
    def tx_usecs_irq(self):
        return self._info.get("tx_usecs_irq")

    @property
    def tx_usecs_low(self):
        return self._info.get("tx_usecs_low")

    @property
    def use_adaptive_rx(self):
        return self._info.get("use_adaptive_rx")

    @property
    def use_adaptive_tx(self):
        return self._info.get("use_adaptive_tx")


class NisporEthtoolRing:
    def __init__(self, info):
        self._info = info

    @property
    def rx(self):
        return self._info.get("rx")

    @property
    def rx_max(self):
        return self._info.get("rx_max")

    @property
    def rx_jumbo(self):
        return self._info.get("rx_jumbo")

    @property
    def rx_jumbo_max(self):
        return self._info.get("rx_jumbo_max")

    @property
    def rx_mini(self):
        return self._info.get("rx_mini")

    @property
    def rx_mini_max(self):
        return self._info.get("rx_mini_max")

    @property
    def tx(self):
        return self._info.get("tx")

    @property
    def tx_max(self):
        return self._info.get("tx_max")


class NisporEthtoolLinkMode:
    def __init__(self, info):
        self._info = info

    @property
    def auto_negotiate(self):
        return self._info["auto_negotiate"]

    @property
    def ours(self):
        return self._info["ours"]

    @property
    def peer(self):
        return self._info.get("peer")

    @property
    def speed(self):
        return self._info["speed"]

    @property
    def duplex(self):
        return self._info["duplex"]

    @property
    def controller_subordinate_cfg(self):
        return self._info.get("controller_subordinate_cfg")

    @property
    def controller_subordinate_state(self):
        return self._info.get("controller_subordinate_state")

    @property
    def lanes(self):
        return self._info.get("lanes")



# Copyright 2020 Red Hat
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .base_iface import NisporBaseIface
from .bond import NisporBond
from .bond import NisporBondSubordinate
from .bridge import NisporBridge
from .bridge import NisporBridgePort
from .tun import NisporTun
from .veth import NisporVeth
from .vlan import NisporVlan
from .vrf import NisporVRF
from .vxlan import NisporVxlan
from .mac_vlan import NisporMacVlan
from .mac_vtap import NisporMacVtap


class NisporIfaceState:
    def __init__(self, info):
        self._ifaces = {}
        if info:
            for iface_name, iface_info in info.items():
                self._ifaces[iface_name] = _iface_info_to_obj(iface_info)

    def __str__(self):
        return f"{self._ifaces}"

    def get(self, iface_name):
        return self._ifaces.get(iface_name)

    def __getitem__(self, iface_name):
        return self._ifaces[iface_name]

    def keys(self):
        for iface in self._ifaces.keys():
            yield iface

    def values(self):
        for iface in self._ifaces.values():
            yield iface


def _iface_info_to_obj(iface_info):
    iface_type = iface_info["iface_type"]
    ctrl_type = iface_info.get("controller_type")
    if iface_type == "bond":
        iface = NisporBond(iface_info)
    elif iface_type == "bridge":
        iface = NisporBridge(iface_info)
    elif iface_type == "tun":
        iface = NisporTun(iface_info)
    elif iface_type == "vlan":
        iface = NisporVlan(iface_info)
    elif iface_type == "vxlan":
        iface = NisporVxlan(iface_info)
    elif iface_type == "veth":
        iface = NisporVeth(iface_info)
    elif iface_type == "vrf":
        iface = NisporVRF(iface_info)
    elif iface_type == "mac_vlan":
        iface = NisporMacVlan(iface_info)
    elif iface_type == "mac_vtap":
        iface = NisporMacVtap(iface_info)
    else:
        iface = NisporBaseIface(iface_info)
    if ctrl_type == "bond":
        iface.subordinate_state = NisporBondSubordinate(iface_info)
    elif ctrl_type == "bridge":
        iface.subordinate_state = NisporBridgePort(iface_info)
    return iface
